@@ -39,6 +39,8 @@
 #endif
 #endif
 
+#include "closeallfds.h"
+
 #define isbitset(v,b) ((v&(b))!=0)
 #define isanybitset(v,b) ((v&(b))!=0)
 #define isallbitset(v,b) ((v&(b))==(b))
@@ -50,26 +52,6 @@ static FILE *logfile = NULL;
 static volatile bool termsignaled = false;
 static char *pidfilename = NULL;
 
-static void closeallfds(int notifd, int logfd, int dirfd, bool warn) {
-	long maxopen;
-	int fd;
-
-	maxopen = sysconf(_SC_OPEN_MAX);
-	if( maxopen > INT_MAX )
-		maxopen = INT_MAX;
-	if( maxopen < 0 )
-		maxopen = 1024;
-	for( fd = 3 ; fd < (int)maxopen ; fd++ ) {
-		if( fd != notifd && fd != logfd && fd != dirfd ) {
-			if( close(fd) == 0 && foreground && warn ) {
-				fprintf(stderr, "File descriptor %d was still open!\n",
-					fd);
-			}
-		}
-	}
-	errno = 0;
-}
-
 static void termsignal(/*@unused@*/ int i __attribute__((unused))) {
 	termsignaled = true;
 }
@@ -246,6 +228,7 @@ static void doaction(const struct lookfor *l, const char *filename) {
 	char **args;
 	pid_t child;
 	size_t i;
+	static const int exceptions[3] = { 0, 1, 2};
 
 	// TODO: what about getting output from the childs and mailing
 	// it somewhere?
@@ -314,7 +297,7 @@ static void doaction(const struct lookfor *l, const char *filename) {
 	}
 	if( logfile != NULL)
 		fclose(logfile);
-	closeallfds(-1, -1, -1, false);
+	closeallopenfds(exceptions, 3);
 	execvp(args[0],args);
 }
 
@@ -704,8 +687,13 @@ int main(int argc, char * const argv[]) {
 		fprintf(pidfile, "%llu", (unsigned long long)getpid());
 		fclose(pidfile);
 	}
-	closeallfds(notifd, (logfile==NULL)?-1:fileno(logfile),
-			    (dir==NULL)?-1:dirfd(dir), true);
+	{
+		const int exceptions[6] = { 0, 1, 2, notifd,
+			(logfile==NULL)?-1:fileno(logfile),
+			(dir==NULL)?-1:dirfd(dir) };
+		closeallopenfds(exceptions, 6);
+		errno = 0;
+	}
 	signal(SIGTERM, termsignal);
 	signal(SIGABRT, termsignal);
 
